# -*- coding: cp1252 -*-
'''
WhatsApp Extractor for Android v1.2
Updated on April 17, 2012
Tested with Whatsapp (Android) 2.7.5613 
Changelog:
V1.3
by Android Fan on April 17, 2012
- corrected linking of offline files (now linking according to media file size)
V1.2
by Android Fan on April 5, 2012
- media files also linked to offline files
- corrected hyperlinks
V1.1
by Android Fan on April 5, 2012
- updated database structure to work on Android (only works for Android version now)
- show contact names
- show smileys
- show images
- link / popup for images, video, audio, gps
- clickable links
V1.0
by Fabio Sangiacomo on December 10, 2011
- first version of Whatsapp Extractor; only working for Iphone version

~~~

WhatsApp Extractor v1.0
Created on Dec 10, 2011

This script opens the msgstore.db WhatsApp SQLite database and creates
a report with the contacts and the conversations for each contact.

(C)opyright 2011 Fabio Sangiacomo <fabio.sangiacomo@digital-forensics.it>
(C)opyright 2012 Android Fan
Released under MIT licence

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

'''

import sys, re, os, string, datetime, time, sqlite3, webbrowser, glob
from optparse import OptionParser

################################################################################
# Chatsession obj definition
class Chatsession:

    # init
    def __init__(self,pkcs,contactname,contactid,contactmsgcount,status):

        # if invalid params are passed, sets attributes to invalid values
        # primary key
        if pkcs == "":
            self.pk_cs = -1
        else:
            self.pk_cs = pkcs
        # contact nick
        if contactname == "":
            self.contact_name = "EMPTY"
        else:
            self.contact_name = contactname
            
        # contact id
        if contactid == "":
            self.contact_id = "EMPTY"
        else:
            self.contact_id = contactid
        # contact msg counter
        if contactmsgcount == "":
            self.contact_msg_count = -1
        else:
            self.contact_msg_count = contactmsgcount

        self.contact_status = status

        # chat session messages
        self.msg_list = []

    # comparison operator
    def __cmp__(self, other):
        if self.pk_cs == other.pk_cs:
                return 0
        return 1

################################################################################
# Message obj definition
class Message:

    # init
    def __init__(self,pkmsg,msgdate,text,contactfrom,contactid,contactto,fromme,status,mediaurl,mediawatype,mediasize,latitude,longitude):

        # if invalid params are passed, sets attributes to invalid values
        # primary key
        if pkmsg == "":
            self.pk_msg = -1
        else:
            self.pk_msg = pkmsg
        # "from me" flag
        if fromme == "":
            self.from_me = -1
        else:
            self.from_me = fromme
        # message timestamp
        if msgdate == "":
            self.msg_date = "N/A"
        else:
            msgdate = str(msgdate)
            #cut last 3 digits (microseconds)
            msgdate = msgdate[:-3]
            self.msg_date = datetime.datetime.fromtimestamp(int(msgdate))
        # message text
        if text == "":
            self.msg_text = "EMPTY"
        else:
            self.msg_text = text
        # message sender
        if contactfrom == "":
            self.contact_from = contactid
        else:
            self.contact_from = contactfrom
        if fromme == "1":
            self.contact_from = "me"
        # message receiver
        if contactto == "":
            self.contact_to = ""
        else:
            self.contact_to = contactto
        # media
        self.media_url = mediaurl
        self.media_wa_type = mediawatype
        self.media_size = mediasize
        #status
        self.status = status
        #GPS
        self.latitude = latitude
        self.longitude = longitude

    # comparison operator
    def __cmp__(self, other):
        if self.pk_msg == other.pk_msg:
                return 0
        return 1
    
################################################################################
#Function Smileys konvertieren
def convertsmileys (text):
    newtext = str(text)

    newtext = newtext.replace(u'\ue415', '<img src="emoji/e415.png" />')
    newtext = newtext.replace(u'\ue056', '<img src="emoji/e056.png" />')
    newtext = newtext.replace(u'\ue057', '<img src="emoji/e057.png" />')
    newtext = newtext.replace(u'\ue414', '<img src="emoji/e414.png" />')
    newtext = newtext.replace(u'\ue405', '<img src="emoji/e405.png" />')
    newtext = newtext.replace(u'\ue106', '<img src="emoji/e106.png" />')
    newtext = newtext.replace(u'\ue418', '<img src="emoji/e418.png" />')

    newtext = newtext.replace(u'\ue417', '<img src="emoji/e417.png" />')
    newtext = newtext.replace(u'\ue40d', '<img src="emoji/e40d.png" />')
    newtext = newtext.replace(u'\ue40a', '<img src="emoji/e40a.png" />')
    newtext = newtext.replace(u'\ue404', '<img src="emoji/e404.png" />')
    newtext = newtext.replace(u'\ue105', '<img src="emoji/e105.png" />')
    newtext = newtext.replace(u'\ue409', '<img src="emoji/e409.png" />')
    newtext = newtext.replace(u'\ue40e', '<img src="emoji/e40e.png" />')

    newtext = newtext.replace(u'\ue402', '<img src="emoji/e402.png" />')
    newtext = newtext.replace(u'\ue108', '<img src="emoji/e108.png" />')
    newtext = newtext.replace(u'\ue403', '<img src="emoji/e403.png" />')
    newtext = newtext.replace(u'\ue058', '<img src="emoji/e058.png" />')
    newtext = newtext.replace(u'\ue407', '<img src="emoji/e407.png" />')
    newtext = newtext.replace(u'\ue401', '<img src="emoji/e401.png" />')
    newtext = newtext.replace(u'\ue40f', '<img src="emoji/e40f.png" />')

    newtext = newtext.replace(u'\ue40b', '<img src="emoji/e40b.png" />')
    newtext = newtext.replace(u'\ue406', '<img src="emoji/e406.png" />')
    newtext = newtext.replace(u'\ue413', '<img src="emoji/e413.png" />')
    newtext = newtext.replace(u'\ue411', '<img src="emoji/e411.png" />')
    newtext = newtext.replace(u'\ue412', '<img src="emoji/e412.png" />')
    newtext = newtext.replace(u'\ue410', '<img src="emoji/e410.png" />')
    newtext = newtext.replace(u'\ue107', '<img src="emoji/e107.png" />')

    newtext = newtext.replace(u'\ue059', '<img src="emoji/e059.png" />')
    newtext = newtext.replace(u'\ue416', '<img src="emoji/e416.png" />')
    newtext = newtext.replace(u'\ue408', '<img src="emoji/e408.png" />')
    newtext = newtext.replace(u'\ue40c', '<img src="emoji/e40c.png" />')
    newtext = newtext.replace(u'\ue11a', '<img src="emoji/e11a.png" />')
    newtext = newtext.replace(u'\ue10c', '<img src="emoji/e10c.png" />')
    newtext = newtext.replace(u'\ue32c', '<img src="emoji/e32c.png" />')

    newtext = newtext.replace(u'\ue32a', '<img src="emoji/e32a.png" />')
    newtext = newtext.replace(u'\ue32d', '<img src="emoji/e32d.png" />')
    newtext = newtext.replace(u'\ue328', '<img src="emoji/e328.png" />')
    newtext = newtext.replace(u'\ue32b', '<img src="emoji/e32b.png" />')
    newtext = newtext.replace(u'\ue022', '<img src="emoji/e022.png" />')
    newtext = newtext.replace(u'\ue023', '<img src="emoji/e023.png" />')
    newtext = newtext.replace(u'\ue327', '<img src="emoji/e327.png" />')

    newtext = newtext.replace(u'\ue329', '<img src="emoji/e329.png" />')
    newtext = newtext.replace(u'\ue32e', '<img src="emoji/e32e.png" />')
    newtext = newtext.replace(u'\ue32f', '<img src="emoji/e32f.png" />')
    newtext = newtext.replace(u'\ue335', '<img src="emoji/e335.png" />')
    newtext = newtext.replace(u'\ue334', '<img src="emoji/e334.png" />')
    newtext = newtext.replace(u'\ue021', '<img src="emoji/e021.png" />')
    newtext = newtext.replace(u'\ue337', '<img src="emoji/e337.png" />')

    newtext = newtext.replace(u'\ue020', '<img src="emoji/e020.png" />')
    newtext = newtext.replace(u'\ue336', '<img src="emoji/e336.png" />')
    newtext = newtext.replace(u'\ue13c', '<img src="emoji/e13c.png" />')
    newtext = newtext.replace(u'\ue330', '<img src="emoji/e330.png" />')
    newtext = newtext.replace(u'\ue331', '<img src="emoji/e331.png" />')
    newtext = newtext.replace(u'\ue326', '<img src="emoji/e326.png" />')
    newtext = newtext.replace(u'\ue03e', '<img src="emoji/e03e.png" />')

    newtext = newtext.replace(u'\ue11d', '<img src="emoji/e11d.png" />')
    newtext = newtext.replace(u'\ue05a', '<img src="emoji/e05a.png" />')
    newtext = newtext.replace(u'\ue00e', '<img src="emoji/e00e.png" />')
    newtext = newtext.replace(u'\ue421', '<img src="emoji/e421.png" />')
    newtext = newtext.replace(u'\ue420', '<img src="emoji/e420.png" />')
    newtext = newtext.replace(u'\ue00d', '<img src="emoji/e00d.png" />')
    newtext = newtext.replace(u'\ue010', '<img src="emoji/e010.png" />')

    newtext = newtext.replace(u'\ue011', '<img src="emoji/e011.png" />')
    newtext = newtext.replace(u'\ue41e', '<img src="emoji/e41e.png" />')
    newtext = newtext.replace(u'\ue012', '<img src="emoji/e012.png" />')
    newtext = newtext.replace(u'\ue422', '<img src="emoji/e422.png" />')
    newtext = newtext.replace(u'\ue22e', '<img src="emoji/e22e.png" />')
    newtext = newtext.replace(u'\ue22f', '<img src="emoji/e22f.png" />')
    newtext = newtext.replace(u'\ue231', '<img src="emoji/e231.png" />')

    newtext = newtext.replace(u'\ue230', '<img src="emoji/e230.png" />')
    newtext = newtext.replace(u'\ue427', '<img src="emoji/e427.png" />')
    newtext = newtext.replace(u'\ue41d', '<img src="emoji/e41d.png" />')
    newtext = newtext.replace(u'\ue00f', '<img src="emoji/e00f.png" />')
    newtext = newtext.replace(u'\ue41f', '<img src="emoji/e41f.png" />')
    newtext = newtext.replace(u'\ue14c', '<img src="emoji/e14c.png" />')
    newtext = newtext.replace(u'\ue201', '<img src="emoji/e201.png" />')

    newtext = newtext.replace(u'\ue115', '<img src="emoji/e115.png" />')
    newtext = newtext.replace(u'\ue428', '<img src="emoji/e428.png" />')
    newtext = newtext.replace(u'\ue51f', '<img src="emoji/e51f.png" />')
    newtext = newtext.replace(u'\ue429', '<img src="emoji/e429.png" />')
    newtext = newtext.replace(u'\ue424', '<img src="emoji/e424.png" />')
    newtext = newtext.replace(u'\ue423', '<img src="emoji/e423.png" />')
    newtext = newtext.replace(u'\ue253', '<img src="emoji/e253.png" />')

    newtext = newtext.replace(u'\ue426', '<img src="emoji/e426.png" />')
    newtext = newtext.replace(u'\ue111', '<img src="emoji/e111.png" />')
    newtext = newtext.replace(u'\ue425', '<img src="emoji/e425.png" />')
    newtext = newtext.replace(u'\ue31e', '<img src="emoji/e31e.png" />')
    newtext = newtext.replace(u'\ue31f', '<img src="emoji/e31f.png" />')
    newtext = newtext.replace(u'\ue31d', '<img src="emoji/e31d.png" />')
    newtext = newtext.replace(u'\ue001', '<img src="emoji/e001.png" />')

    newtext = newtext.replace(u'\ue002', '<img src="emoji/e002.png" />')
    newtext = newtext.replace(u'\ue005', '<img src="emoji/e005.png" />')
    newtext = newtext.replace(u'\ue004', '<img src="emoji/e004.png" />')
    newtext = newtext.replace(u'\ue51a', '<img src="emoji/e51a.png" />')
    newtext = newtext.replace(u'\ue519', '<img src="emoji/e519.png" />')
    newtext = newtext.replace(u'\ue518', '<img src="emoji/e518.png" />')
    newtext = newtext.replace(u'\ue515', '<img src="emoji/e515.png" />')

    newtext = newtext.replace(u'\ue516', '<img src="emoji/e516.png" />')
    newtext = newtext.replace(u'\ue517', '<img src="emoji/e517.png" />')
    newtext = newtext.replace(u'\ue51b', '<img src="emoji/e51b.png" />')
    newtext = newtext.replace(u'\ue152', '<img src="emoji/e152.png" />')
    newtext = newtext.replace(u'\ue04e', '<img src="emoji/e04e.png" />')
    newtext = newtext.replace(u'\ue51c', '<img src="emoji/e51c.png" />')
    newtext = newtext.replace(u'\ue51e', '<img src="emoji/e51e.png" />')

    newtext = newtext.replace(u'\ue11c', '<img src="emoji/e11c.png" />')
    newtext = newtext.replace(u'\ue536', '<img src="emoji/e536.png" />')
    newtext = newtext.replace(u'\ue003', '<img src="emoji/e003.png" />')
    newtext = newtext.replace(u'\ue41c', '<img src="emoji/e41c.png" />')
    newtext = newtext.replace(u'\ue41b', '<img src="emoji/e41b.png" />')
    newtext = newtext.replace(u'\ue419', '<img src="emoji/e419.png" />')
    newtext = newtext.replace(u'\ue41a', '<img src="emoji/e41a.png" />')

    newtext = newtext.replace(u'\ue04a', '<img src="emoji/e04a.png" />')
    newtext = newtext.replace(u'\ue04b', '<img src="emoji/e04b.png" />')
    newtext = newtext.replace(u'\ue049', '<img src="emoji/e049.png" />')
    newtext = newtext.replace(u'\ue048', '<img src="emoji/e048.png" />')
    newtext = newtext.replace(u'\ue04c', '<img src="emoji/e04c.png" />')
    newtext = newtext.replace(u'\ue13d', '<img src="emoji/e13d.png" />')
    newtext = newtext.replace(u'\ue443', '<img src="emoji/e443.png" />')

    newtext = newtext.replace(u'\ue43e', '<img src="emoji/e43e.png" />')
    newtext = newtext.replace(u'\ue04f', '<img src="emoji/e04f.png" />')
    newtext = newtext.replace(u'\ue052', '<img src="emoji/e052.png" />')
    newtext = newtext.replace(u'\ue053', '<img src="emoji/e053.png" />')
    newtext = newtext.replace(u'\ue524', '<img src="emoji/e524.png" />')
    newtext = newtext.replace(u'\ue52c', '<img src="emoji/e52c.png" />')
    newtext = newtext.replace(u'\ue52a', '<img src="emoji/e52a.png" />')

    newtext = newtext.replace(u'\ue531', '<img src="emoji/e531.png" />')
    newtext = newtext.replace(u'\ue050', '<img src="emoji/e050.png" />')
    newtext = newtext.replace(u'\ue527', '<img src="emoji/e527.png" />')
    newtext = newtext.replace(u'\ue051', '<img src="emoji/e051.png" />')
    newtext = newtext.replace(u'\ue10b', '<img src="emoji/e10b.png" />')
    newtext = newtext.replace(u'\ue52b', '<img src="emoji/e52b.png" />')
    newtext = newtext.replace(u'\ue52f', '<img src="emoji/e52f.png" />')

    newtext = newtext.replace(u'\ue528', '<img src="emoji/e528.png" />')
    newtext = newtext.replace(u'\ue01a', '<img src="emoji/e01a.png" />')
    newtext = newtext.replace(u'\ue134', '<img src="emoji/e134.png" />')
    newtext = newtext.replace(u'\ue530', '<img src="emoji/e530.png" />')
    newtext = newtext.replace(u'\ue529', '<img src="emoji/e529.png" />')
    newtext = newtext.replace(u'\ue526', '<img src="emoji/e526.png" />')
    newtext = newtext.replace(u'\ue52d', '<img src="emoji/e52d.png" />')

    newtext = newtext.replace(u'\ue521', '<img src="emoji/e521.png" />')
    newtext = newtext.replace(u'\ue523', '<img src="emoji/e523.png" />')
    newtext = newtext.replace(u'\ue52e', '<img src="emoji/e52e.png" />')
    newtext = newtext.replace(u'\ue055', '<img src="emoji/e055.png" />')
    newtext = newtext.replace(u'\ue525', '<img src="emoji/e525.png" />')
    newtext = newtext.replace(u'\ue10a', '<img src="emoji/e10a.png" />')
    newtext = newtext.replace(u'\ue109', '<img src="emoji/e109.png" />')

    newtext = newtext.replace(u'\ue522', '<img src="emoji/e522.png" />')
    newtext = newtext.replace(u'\ue019', '<img src="emoji/e019.png" />')
    newtext = newtext.replace(u'\ue054', '<img src="emoji/e054.png" />')
    newtext = newtext.replace(u'\ue520', '<img src="emoji/e520.png" />')
    newtext = newtext.replace(u'\ue306', '<img src="emoji/e306.png" />')
    newtext = newtext.replace(u'\ue030', '<img src="emoji/e030.png" />')
    newtext = newtext.replace(u'\ue304', '<img src="emoji/e304.png" />')

    newtext = newtext.replace(u'\ue110', '<img src="emoji/e110.png" />')
    newtext = newtext.replace(u'\ue032', '<img src="emoji/e032.png" />')
    newtext = newtext.replace(u'\ue305', '<img src="emoji/e305.png" />')
    newtext = newtext.replace(u'\ue303', '<img src="emoji/e303.png" />')
    newtext = newtext.replace(u'\ue118', '<img src="emoji/e118.png" />')
    newtext = newtext.replace(u'\ue447', '<img src="emoji/e447.png" />')
    newtext = newtext.replace(u'\ue119', '<img src="emoji/e119.png" />')

    newtext = newtext.replace(u'\ue307', '<img src="emoji/e307.png" />')
    newtext = newtext.replace(u'\ue308', '<img src="emoji/e308.png" />')
    newtext = newtext.replace(u'\ue444', '<img src="emoji/e444.png" />')
    newtext = newtext.replace(u'\ue441', '<img src="emoji/e441.png" />')

    newtext = newtext.replace(u'\ue436', '<img src="emoji/e436.png" />')
    newtext = newtext.replace(u'\ue437', '<img src="emoji/e437.png" />')
    newtext = newtext.replace(u'\ue438', '<img src="emoji/e438.png" />')
    newtext = newtext.replace(u'\ue43a', '<img src="emoji/e43a.png" />')
    newtext = newtext.replace(u'\ue439', '<img src="emoji/e439.png" />')
    newtext = newtext.replace(u'\ue43b', '<img src="emoji/e43b.png" />')
    newtext = newtext.replace(u'\ue117', '<img src="emoji/e117.png" />')

    newtext = newtext.replace(u'\ue440', '<img src="emoji/e440.png" />')
    newtext = newtext.replace(u'\ue442', '<img src="emoji/e442.png" />')
    newtext = newtext.replace(u'\ue446', '<img src="emoji/e446.png" />')
    newtext = newtext.replace(u'\ue445', '<img src="emoji/e445.png" />')
    newtext = newtext.replace(u'\ue11b', '<img src="emoji/e11b.png" />')
    newtext = newtext.replace(u'\ue448', '<img src="emoji/e448.png" />')
    newtext = newtext.replace(u'\ue033', '<img src="emoji/e033.png" />')

    newtext = newtext.replace(u'\ue112', '<img src="emoji/e112.png" />')
    newtext = newtext.replace(u'\ue325', '<img src="emoji/e325.png" />')
    newtext = newtext.replace(u'\ue312', '<img src="emoji/e312.png" />')
    newtext = newtext.replace(u'\ue310', '<img src="emoji/e310.png" />')
    newtext = newtext.replace(u'\ue126', '<img src="emoji/e126.png" />')
    newtext = newtext.replace(u'\ue127', '<img src="emoji/e127.png" />')
    newtext = newtext.replace(u'\ue008', '<img src="emoji/e008.png" />')

    newtext = newtext.replace(u'\ue03d', '<img src="emoji/e03d.png" />')
    newtext = newtext.replace(u'\ue00c', '<img src="emoji/e00c.png" />')
    newtext = newtext.replace(u'\ue12a', '<img src="emoji/e12a.png" />')
    newtext = newtext.replace(u'\ue00a', '<img src="emoji/e00a.png" />')
    newtext = newtext.replace(u'\ue00b', '<img src="emoji/e00b.png" />')
    newtext = newtext.replace(u'\ue009', '<img src="emoji/e009.png" />')
    newtext = newtext.replace(u'\ue316', '<img src="emoji/e316.png" />')

    newtext = newtext.replace(u'\ue129', '<img src="emoji/e129.png" />')
    newtext = newtext.replace(u'\ue141', '<img src="emoji/e141.png" />')
    newtext = newtext.replace(u'\ue142', '<img src="emoji/e142.png" />')
    newtext = newtext.replace(u'\ue317', '<img src="emoji/e317.png" />')
    newtext = newtext.replace(u'\ue128', '<img src="emoji/e128.png" />')
    newtext = newtext.replace(u'\ue14b', '<img src="emoji/e14b.png" />')
    newtext = newtext.replace(u'\ue211', '<img src="emoji/e211.png" />')

    newtext = newtext.replace(u'\ue114', '<img src="emoji/e114.png" />')
    newtext = newtext.replace(u'\ue145', '<img src="emoji/e145.png" />')
    newtext = newtext.replace(u'\ue144', '<img src="emoji/e144.png" />')
    newtext = newtext.replace(u'\ue03f', '<img src="emoji/e03f.png" />')
    newtext = newtext.replace(u'\ue313', '<img src="emoji/e313.png" />')
    newtext = newtext.replace(u'\ue116', '<img src="emoji/e116.png" />')
    newtext = newtext.replace(u'\ue10f', '<img src="emoji/e10f.png" />')

    newtext = newtext.replace(u'\ue104', '<img src="emoji/e104.png" />')
    newtext = newtext.replace(u'\ue103', '<img src="emoji/e103.png" />')
    newtext = newtext.replace(u'\ue101', '<img src="emoji/e101.png" />')
    newtext = newtext.replace(u'\ue102', '<img src="emoji/e102.png" />')
    newtext = newtext.replace(u'\ue13f', '<img src="emoji/e13f.png" />')
    newtext = newtext.replace(u'\ue140', '<img src="emoji/e140.png" />')
    newtext = newtext.replace(u'\ue11f', '<img src="emoji/e11f.png" />')

    newtext = newtext.replace(u'\ue12f', '<img src="emoji/e12f.png" />')
    newtext = newtext.replace(u'\ue031', '<img src="emoji/e031.png" />')
    newtext = newtext.replace(u'\ue30e', '<img src="emoji/e30e.png" />')
    newtext = newtext.replace(u'\ue311', '<img src="emoji/e311.png" />')
    newtext = newtext.replace(u'\ue113', '<img src="emoji/e113.png" />')
    newtext = newtext.replace(u'\ue30f', '<img src="emoji/e30f.png" />')
    newtext = newtext.replace(u'\ue13b', '<img src="emoji/e13b.png" />')

    newtext = newtext.replace(u'\ue42b', '<img src="emoji/e42b.png" />')
    newtext = newtext.replace(u'\ue42a', '<img src="emoji/e42a.png" />')
    newtext = newtext.replace(u'\ue018', '<img src="emoji/e018.png" />')
    newtext = newtext.replace(u'\ue016', '<img src="emoji/e016.png" />')
    newtext = newtext.replace(u'\ue015', '<img src="emoji/e015.png" />')
    newtext = newtext.replace(u'\ue014', '<img src="emoji/e014.png" />')
    newtext = newtext.replace(u'\ue42c', '<img src="emoji/e42c.png" />')

    newtext = newtext.replace(u'\ue42d', '<img src="emoji/e42d.png" />')
    newtext = newtext.replace(u'\ue017', '<img src="emoji/e017.png" />')
    newtext = newtext.replace(u'\ue013', '<img src="emoji/e013.png" />')
    newtext = newtext.replace(u'\ue20e', '<img src="emoji/e20e.png" />')
    newtext = newtext.replace(u'\ue20c', '<img src="emoji/e20c.png" />')
    newtext = newtext.replace(u'\ue20f', '<img src="emoji/e20f.png" />')
    newtext = newtext.replace(u'\ue20d', '<img src="emoji/e20d.png" />')

    newtext = newtext.replace(u'\ue131', '<img src="emoji/e131.png" />')
    newtext = newtext.replace(u'\ue12b', '<img src="emoji/e12b.png" />')
    newtext = newtext.replace(u'\ue130', '<img src="emoji/e130.png" />')
    newtext = newtext.replace(u'\ue12d', '<img src="emoji/e12d.png" />')
    newtext = newtext.replace(u'\ue324', '<img src="emoji/e324.png" />')
    newtext = newtext.replace(u'\ue301', '<img src="emoji/e301.png" />')
    newtext = newtext.replace(u'\ue148', '<img src="emoji/e148.png" />')

    newtext = newtext.replace(u'\ue502', '<img src="emoji/e502.png" />')
    newtext = newtext.replace(u'\ue03c', '<img src="emoji/e03c.png" />')
    newtext = newtext.replace(u'\ue30a', '<img src="emoji/e30a.png" />')
    newtext = newtext.replace(u'\ue042', '<img src="emoji/e042.png" />')
    newtext = newtext.replace(u'\ue040', '<img src="emoji/e040.png" />')
    newtext = newtext.replace(u'\ue041', '<img src="emoji/e041.png" />')
    newtext = newtext.replace(u'\ue12c', '<img src="emoji/e12c.png" />')

    newtext = newtext.replace(u'\ue007', '<img src="emoji/e007.png" />')
    newtext = newtext.replace(u'\ue31a', '<img src="emoji/e31a.png" />')
    newtext = newtext.replace(u'\ue13e', '<img src="emoji/e13e.png" />')
    newtext = newtext.replace(u'\ue31b', '<img src="emoji/e31b.png" />')
    newtext = newtext.replace(u'\ue006', '<img src="emoji/e006.png" />')
    newtext = newtext.replace(u'\ue302', '<img src="emoji/e302.png" />')
    newtext = newtext.replace(u'\ue319', '<img src="emoji/e319.png" />')

    newtext = newtext.replace(u'\ue321', '<img src="emoji/e321.png" />')
    newtext = newtext.replace(u'\ue322', '<img src="emoji/e322.png" />')
    newtext = newtext.replace(u'\ue314', '<img src="emoji/e314.png" />')
    newtext = newtext.replace(u'\ue503', '<img src="emoji/e503.png" />')
    newtext = newtext.replace(u'\ue10e', '<img src="emoji/e10e.png" />')
    newtext = newtext.replace(u'\ue318', '<img src="emoji/e318.png" />')
    newtext = newtext.replace(u'\ue43c', '<img src="emoji/e43c.png" />')

    newtext = newtext.replace(u'\ue11e', '<img src="emoji/e11e.png" />')
    newtext = newtext.replace(u'\ue323', '<img src="emoji/e323.png" />')
    newtext = newtext.replace(u'\ue31c', '<img src="emoji/e31c.png" />')
    newtext = newtext.replace(u'\ue034', '<img src="emoji/e034.png" />')
    newtext = newtext.replace(u'\ue035', '<img src="emoji/e035.png" />')
    newtext = newtext.replace(u'\ue045', '<img src="emoji/e045.png" />')
    newtext = newtext.replace(u'\ue338', '<img src="emoji/e338.png" />')

    newtext = newtext.replace(u'\ue047', '<img src="emoji/e047.png" />')
    newtext = newtext.replace(u'\ue30c', '<img src="emoji/e30c.png" />')
    newtext = newtext.replace(u'\ue044', '<img src="emoji/e044.png" />')
    newtext = newtext.replace(u'\ue30b', '<img src="emoji/e30b.png" />')
    newtext = newtext.replace(u'\ue043', '<img src="emoji/e043.png" />')
    newtext = newtext.replace(u'\ue120', '<img src="emoji/e120.png" />')
    newtext = newtext.replace(u'\ue33b', '<img src="emoji/e33b.png" />')

    newtext = newtext.replace(u'\ue33f', '<img src="emoji/e33f.png" />')
    newtext = newtext.replace(u'\ue341', '<img src="emoji/e341.png" />')
    newtext = newtext.replace(u'\ue34c', '<img src="emoji/e34c.png" />')
    newtext = newtext.replace(u'\ue344', '<img src="emoji/e344.png" />')
    newtext = newtext.replace(u'\ue342', '<img src="emoji/e342.png" />')
    newtext = newtext.replace(u'\ue33d', '<img src="emoji/e33d.png" />')
    newtext = newtext.replace(u'\ue33e', '<img src="emoji/e33e.png" />')

    newtext = newtext.replace(u'\ue340', '<img src="emoji/e340.png" />')
    newtext = newtext.replace(u'\ue34d', '<img src="emoji/e34d.png" />')
    newtext = newtext.replace(u'\ue339', '<img src="emoji/e339.png" />')
    newtext = newtext.replace(u'\ue147', '<img src="emoji/e147.png" />')
    newtext = newtext.replace(u'\ue343', '<img src="emoji/e343.png" />')
    newtext = newtext.replace(u'\ue33c', '<img src="emoji/e33c.png" />')
    newtext = newtext.replace(u'\ue33a', '<img src="emoji/e33a.png" />')

    newtext = newtext.replace(u'\ue43f', '<img src="emoji/e43f.png" />')
    newtext = newtext.replace(u'\ue34b', '<img src="emoji/e34b.png" />')
    newtext = newtext.replace(u'\ue046', '<img src="emoji/e046.png" />')
    newtext = newtext.replace(u'\ue345', '<img src="emoji/e345.png" />')
    newtext = newtext.replace(u'\ue346', '<img src="emoji/e346.png" />')
    newtext = newtext.replace(u'\ue348', '<img src="emoji/e348.png" />')
    newtext = newtext.replace(u'\ue347', '<img src="emoji/e347.png" />')

    newtext = newtext.replace(u'\ue34a', '<img src="emoji/e34a.png" />')
    newtext = newtext.replace(u'\ue349', '<img src="emoji/e349.png" />')

    newtext = newtext.replace(u'\ue036', '<img src="emoji/e036.png" />')
    newtext = newtext.replace(u'\ue157', '<img src="emoji/e157.png" />')
    newtext = newtext.replace(u'\ue038', '<img src="emoji/e038.png" />')
    newtext = newtext.replace(u'\ue153', '<img src="emoji/e153.png" />')
    newtext = newtext.replace(u'\ue155', '<img src="emoji/e155.png" />')
    newtext = newtext.replace(u'\ue14d', '<img src="emoji/e14d.png" />')
    newtext = newtext.replace(u'\ue156', '<img src="emoji/e156.png" />')

    newtext = newtext.replace(u'\ue501', '<img src="emoji/e501.png" />')
    newtext = newtext.replace(u'\ue158', '<img src="emoji/e158.png" />')
    newtext = newtext.replace(u'\ue43d', '<img src="emoji/e43d.png" />')
    newtext = newtext.replace(u'\ue037', '<img src="emoji/e037.png" />')
    newtext = newtext.replace(u'\ue504', '<img src="emoji/e504.png" />')
    newtext = newtext.replace(u'\ue44a', '<img src="emoji/e44a.png" />')
    newtext = newtext.replace(u'\ue146', '<img src="emoji/e146.png" />')

    newtext = newtext.replace(u'\ue50a', '<img src="emoji/e50a.png" />')
    newtext = newtext.replace(u'\ue505', '<img src="emoji/e505.png" />')
    newtext = newtext.replace(u'\ue506', '<img src="emoji/e506.png" />')
    newtext = newtext.replace(u'\ue122', '<img src="emoji/e122.png" />')
    newtext = newtext.replace(u'\ue508', '<img src="emoji/e508.png" />')
    newtext = newtext.replace(u'\ue509', '<img src="emoji/e509.png" />')
    newtext = newtext.replace(u'\ue03b', '<img src="emoji/e03b.png" />')

    newtext = newtext.replace(u'\ue04d', '<img src="emoji/e04d.png" />')
    newtext = newtext.replace(u'\ue449', '<img src="emoji/e449.png" />')
    newtext = newtext.replace(u'\ue44b', '<img src="emoji/e44b.png" />')
    newtext = newtext.replace(u'\ue51d', '<img src="emoji/e51d.png" />')
    newtext = newtext.replace(u'\ue44c', '<img src="emoji/e44c.png" />')
    newtext = newtext.replace(u'\ue124', '<img src="emoji/e124.png" />')
    newtext = newtext.replace(u'\ue121', '<img src="emoji/e121.png" />')

    newtext = newtext.replace(u'\ue433', '<img src="emoji/e433.png" />')
    newtext = newtext.replace(u'\ue202', '<img src="emoji/e202.png" />')
    newtext = newtext.replace(u'\ue135', '<img src="emoji/e135.png" />')
    newtext = newtext.replace(u'\ue01c', '<img src="emoji/e01c.png" />')
    newtext = newtext.replace(u'\ue01d', '<img src="emoji/e01d.png" />')
    newtext = newtext.replace(u'\ue10d', '<img src="emoji/e10d.png" />')
    newtext = newtext.replace(u'\ue136', '<img src="emoji/e136.png" />')

    newtext = newtext.replace(u'\ue42e', '<img src="emoji/e42e.png" />')
    newtext = newtext.replace(u'\ue01b', '<img src="emoji/e01b.png" />')
    newtext = newtext.replace(u'\ue15a', '<img src="emoji/e15a.png" />')
    newtext = newtext.replace(u'\ue159', '<img src="emoji/e159.png" />')
    newtext = newtext.replace(u'\ue432', '<img src="emoji/e432.png" />')
    newtext = newtext.replace(u'\ue430', '<img src="emoji/e430.png" />')
    newtext = newtext.replace(u'\ue431', '<img src="emoji/e431.png" />')

    newtext = newtext.replace(u'\ue42f', '<img src="emoji/e42f.png" />')
    newtext = newtext.replace(u'\ue01e', '<img src="emoji/e01e.png" />')
    newtext = newtext.replace(u'\ue039', '<img src="emoji/e039.png" />')
    newtext = newtext.replace(u'\ue435', '<img src="emoji/e435.png" />')
    newtext = newtext.replace(u'\ue01f', '<img src="emoji/e01f.png" />')
    newtext = newtext.replace(u'\ue125', '<img src="emoji/e125.png" />')
    newtext = newtext.replace(u'\ue03a', '<img src="emoji/e03a.png" />')

    newtext = newtext.replace(u'\ue14e', '<img src="emoji/e14e.png" />')
    newtext = newtext.replace(u'\ue252', '<img src="emoji/e252.png" />')
    newtext = newtext.replace(u'\ue137', '<img src="emoji/e137.png" />')
    newtext = newtext.replace(u'\ue209', '<img src="emoji/e209.png" />')
    newtext = newtext.replace(u'\ue154', '<img src="emoji/e154.png" />')
    newtext = newtext.replace(u'\ue133', '<img src="emoji/e133.png" />')
    newtext = newtext.replace(u'\ue150', '<img src="emoji/e150.png" />')

    newtext = newtext.replace(u'\ue320', '<img src="emoji/e320.png" />')
    newtext = newtext.replace(u'\ue123', '<img src="emoji/e123.png" />')
    newtext = newtext.replace(u'\ue132', '<img src="emoji/e132.png" />')
    newtext = newtext.replace(u'\ue143', '<img src="emoji/e143.png" />')
    newtext = newtext.replace(u'\ue50b', '<img src="emoji/e50b.png" />')
    newtext = newtext.replace(u'\ue514', '<img src="emoji/e514.png" />')
    newtext = newtext.replace(u'\ue513', '<img src="emoji/e513.png" />')

    newtext = newtext.replace(u'\ue50c', '<img src="emoji/e50c.png" />')
    newtext = newtext.replace(u'\ue50d', '<img src="emoji/e50d.png" />')
    newtext = newtext.replace(u'\ue511', '<img src="emoji/e511.png" />')
    newtext = newtext.replace(u'\ue50f', '<img src="emoji/e50f.png" />')
    newtext = newtext.replace(u'\ue512', '<img src="emoji/e512.png" />')
    newtext = newtext.replace(u'\ue510', '<img src="emoji/e510.png" />')
    newtext = newtext.replace(u'\ue50e', '<img src="emoji/e50e.png" />')

    newtext = newtext.replace(u'\ue21c', '<img src="emoji/e21c.png" />')
    newtext = newtext.replace(u'\ue21d', '<img src="emoji/e21d.png" />')
    newtext = newtext.replace(u'\ue21e', '<img src="emoji/e21e.png" />')
    newtext = newtext.replace(u'\ue21f', '<img src="emoji/e21f.png" />')
    newtext = newtext.replace(u'\ue220', '<img src="emoji/e220.png" />')
    newtext = newtext.replace(u'\ue221', '<img src="emoji/e221.png" />')
    newtext = newtext.replace(u'\ue222', '<img src="emoji/e222.png" />')

    newtext = newtext.replace(u'\ue223', '<img src="emoji/e223.png" />')
    newtext = newtext.replace(u'\ue224', '<img src="emoji/e224.png" />')
    newtext = newtext.replace(u'\ue225', '<img src="emoji/e225.png" />')
    newtext = newtext.replace(u'\ue210', '<img src="emoji/e210.png" />')
    newtext = newtext.replace(u'\ue232', '<img src="emoji/e232.png" />')
    newtext = newtext.replace(u'\ue233', '<img src="emoji/e233.png" />')
    newtext = newtext.replace(u'\ue235', '<img src="emoji/e235.png" />')

    newtext = newtext.replace(u'\ue234', '<img src="emoji/e234.png" />')
    newtext = newtext.replace(u'\ue236', '<img src="emoji/e236.png" />')
    newtext = newtext.replace(u'\ue237', '<img src="emoji/e237.png" />')
    newtext = newtext.replace(u'\ue238', '<img src="emoji/e238.png" />')
    newtext = newtext.replace(u'\ue239', '<img src="emoji/e239.png" />')
    newtext = newtext.replace(u'\ue23b', '<img src="emoji/e23b.png" />')
    newtext = newtext.replace(u'\ue23a', '<img src="emoji/e23a.png" />')

    newtext = newtext.replace(u'\ue23d', '<img src="emoji/e23d.png" />')
    newtext = newtext.replace(u'\ue23c', '<img src="emoji/e23c.png" />')
    newtext = newtext.replace(u'\ue24d', '<img src="emoji/e24d.png" />')
    newtext = newtext.replace(u'\ue212', '<img src="emoji/e212.png" />')
    newtext = newtext.replace(u'\ue24c', '<img src="emoji/e24c.png" />')
    newtext = newtext.replace(u'\ue213', '<img src="emoji/e213.png" />')
    newtext = newtext.replace(u'\ue214', '<img src="emoji/e214.png" />')

    newtext = newtext.replace(u'\ue507', '<img src="emoji/e507.png" />')
    newtext = newtext.replace(u'\ue203', '<img src="emoji/e203.png" />')
    newtext = newtext.replace(u'\ue20b', '<img src="emoji/e20b.png" />')
    newtext = newtext.replace(u'\ue22a', '<img src="emoji/e22a.png" />')
    newtext = newtext.replace(u'\ue22b', '<img src="emoji/e22b.png" />')
    newtext = newtext.replace(u'\ue226', '<img src="emoji/e226.png" />')
    newtext = newtext.replace(u'\ue227', '<img src="emoji/e227.png" />')

    newtext = newtext.replace(u'\ue22c', '<img src="emoji/e22c.png" />')
    newtext = newtext.replace(u'\ue22d', '<img src="emoji/e22d.png" />')
    newtext = newtext.replace(u'\ue215', '<img src="emoji/e215.png" />')
    newtext = newtext.replace(u'\ue216', '<img src="emoji/e216.png" />')
    newtext = newtext.replace(u'\ue217', '<img src="emoji/e217.png" />')
    newtext = newtext.replace(u'\ue218', '<img src="emoji/e218.png" />')
    newtext = newtext.replace(u'\ue228', '<img src="emoji/e228.png" />')

    newtext = newtext.replace(u'\ue151', '<img src="emoji/e151.png" />')
    newtext = newtext.replace(u'\ue138', '<img src="emoji/e138.png" />')
    newtext = newtext.replace(u'\ue139', '<img src="emoji/e139.png" />')
    newtext = newtext.replace(u'\ue13a', '<img src="emoji/e13a.png" />')
    newtext = newtext.replace(u'\ue208', '<img src="emoji/e208.png" />')
    newtext = newtext.replace(u'\ue14f', '<img src="emoji/e14f.png" />')
    newtext = newtext.replace(u'\ue20a', '<img src="emoji/e20a.png" />')

    newtext = newtext.replace(u'\ue434', '<img src="emoji/e434.png" />')
    newtext = newtext.replace(u'\ue309', '<img src="emoji/e309.png" />')
    newtext = newtext.replace(u'\ue315', '<img src="emoji/e315.png" />')
    newtext = newtext.replace(u'\ue30d', '<img src="emoji/e30d.png" />')
    newtext = newtext.replace(u'\ue207', '<img src="emoji/e207.png" />')
    newtext = newtext.replace(u'\ue229', '<img src="emoji/e229.png" />')
    newtext = newtext.replace(u'\ue206', '<img src="emoji/e206.png" />')

    newtext = newtext.replace(u'\ue205', '<img src="emoji/e205.png" />')
    newtext = newtext.replace(u'\ue204', '<img src="emoji/e204.png" />')
    newtext = newtext.replace(u'\ue12e', '<img src="emoji/e12e.png" />')
    newtext = newtext.replace(u'\ue250', '<img src="emoji/e250.png" />')
    newtext = newtext.replace(u'\ue251', '<img src="emoji/e251.png" />')
    newtext = newtext.replace(u'\ue14a', '<img src="emoji/e14a.png" />')
    newtext = newtext.replace(u'\ue149', '<img src="emoji/e149.png" />')

    newtext = newtext.replace(u'\ue23f', '<img src="emoji/e23f.png" />')
    newtext = newtext.replace(u'\ue240', '<img src="emoji/e240.png" />')
    newtext = newtext.replace(u'\ue241', '<img src="emoji/e241.png" />')
    newtext = newtext.replace(u'\ue242', '<img src="emoji/e242.png" />')
    newtext = newtext.replace(u'\ue243', '<img src="emoji/e243.png" />')
    newtext = newtext.replace(u'\ue244', '<img src="emoji/e244.png" />')
    newtext = newtext.replace(u'\ue245', '<img src="emoji/e245.png" />')

    newtext = newtext.replace(u'\ue246', '<img src="emoji/e246.png" />')
    newtext = newtext.replace(u'\ue247', '<img src="emoji/e247.png" />')
    newtext = newtext.replace(u'\ue248', '<img src="emoji/e248.png" />')
    newtext = newtext.replace(u'\ue249', '<img src="emoji/e249.png" />')
    newtext = newtext.replace(u'\ue24a', '<img src="emoji/e24a.png" />')
    newtext = newtext.replace(u'\ue24b', '<img src="emoji/e24b.png" />')
    newtext = newtext.replace(u'\ue23e', '<img src="emoji/e23e.png" />')

    newtext = newtext.replace(u'\ue532', '<img src="emoji/e532.png" />')
    newtext = newtext.replace(u'\ue533', '<img src="emoji/e533.png" />')
    newtext = newtext.replace(u'\ue534', '<img src="emoji/e534.png" />')
    newtext = newtext.replace(u'\ue535', '<img src="emoji/e535.png" />')
    newtext = newtext.replace(u'\ue21a', '<img src="emoji/e21a.png" />')
    newtext = newtext.replace(u'\ue219', '<img src="emoji/e219.png" />')
    newtext = newtext.replace(u'\ue21b', '<img src="emoji/e21b.png" />')

    newtext = newtext.replace(u'\ue02f', '<img src="emoji/e02f.png" />')
    newtext = newtext.replace(u'\ue024', '<img src="emoji/e024.png" />')
    newtext = newtext.replace(u'\ue025', '<img src="emoji/e025.png" />')
    newtext = newtext.replace(u'\ue026', '<img src="emoji/e026.png" />')
    newtext = newtext.replace(u'\ue027', '<img src="emoji/e027.png" />')
    newtext = newtext.replace(u'\ue028', '<img src="emoji/e028.png" />')
    newtext = newtext.replace(u'\ue029', '<img src="emoji/e029.png" />')

    newtext = newtext.replace(u'\ue02a', '<img src="emoji/e02a.png" />')
    newtext = newtext.replace(u'\ue02b', '<img src="emoji/e02b.png" />')
    newtext = newtext.replace(u'\ue02c', '<img src="emoji/e02c.png" />')
    newtext = newtext.replace(u'\ue02d', '<img src="emoji/e02d.png" />')
    newtext = newtext.replace(u'\ue02e', '<img src="emoji/e02e.png" />')
    newtext = newtext.replace(u'\ue332', '<img src="emoji/e332.png" />')
    newtext = newtext.replace(u'\ue333', '<img src="emoji/e333.png" />')

    newtext = newtext.replace(u'\ue24e', '<img src="emoji/e24e.png" />')
    newtext = newtext.replace(u'\ue24f', '<img src="emoji/e24f.png" />')
    newtext = newtext.replace(u'\ue537', '<img src="emoji/e537.png" />')

    return newtext

#Functions for Find Offline File
def filelistonce (folder, date):
    flistnames = glob.glob( os.path.join(folder, '*'+date+'*') )
    flistsizes = []
    for i in range(len(flistnames)):
        statinfo = os.stat(flistnames[i])
        fsize = statinfo.st_size
        flistsizes.append(fsize)
    flist = [flistnames, flistsizes]
    return flist
def filelist (type, date):
    if type == 'IMG':
        folder = 'Media/WhatsApp Images/'
        if not date in flistimg:
            flistimg[date] = filelistonce (folder, date)
        flist = flistimg[date]
    if type == 'AUD':
        folder = 'Media/WhatsApp Audio/'
        if not date in flistaud:
            flistaud[date] = filelistonce (folder, date)
        flist = flistaud[date]
    if type == 'VID':
        folder = 'Media/WhatsApp Video/'
        if not date in flistvid:
            flistvid[date] = filelistonce (folder, date)
        flist = flistvid[date]
    return folder, flist
def findfile (type, size, date, additionaldays):
    date = str(date)
    fname = ''
    z = 0
    while fname == '':
        z = z + 1
        #use or create list of all media files of that type and that date
        folder, flist = filelist (type, date)
        #search the file with the given size
        try:
            fname = flist [0] [ flist[1].index(size) ]
        except:
            fname = ''          
        if fname == '':
            if z >= 1 + additionaldays:
                #if file is not found for "today", then try for "tomorrow" and "the day after tomorrow". If it's still not found, then just link the media folder
                fname = folder
            else:
                timestamptoday = int(str(time.mktime(datetime.datetime.strptime(date, "%Y%m%d").timetuple()))[:-2])
                timestamptomorrow = timestamptoday + 86400
                tomorrow = str(datetime.datetime.fromtimestamp(timestamptomorrow))[:10]
                tomorrow = tomorrow.replace("-","")
                date = tomorrow
   #return the file name
    return fname

            
################################################################################
# MAIN
def main(argv):

    reload(sys)
    sys.setdefaultencoding( "utf-8" )
    #sys.setdefaultencoding( "cp1252" )

    chat_session_list = []

    # parser options
    parser = OptionParser()
    parser.add_option("-i", "--infile", dest="infile", help="input 'msgstore.db' file to scan")
    parser.add_option("-w", "--wafile", dest="wafile", help="input 'wa.db' file to scan")
    (options, args) = parser.parse_args()

    # checks for the input file
    if options.infile is None:
        parser.print_help()
        sys.exit(1)
    if not os.path.exists(options.infile):
        print '"{}" file is not found!'.format(options.infile)
        sys.exit(1)

    # checks for the wadb file
    if options.wafile is None:
        parser.print_help()
        sys.exit(1)
    if not os.path.exists(options.wafile):
        print '"{}" file is not found!'.format(options.wafile)
        sys.exit(1)

    # connects to the database
    msgstore = sqlite3.connect(options.infile)
    c = msgstore.cursor()
    wastore = sqlite3.connect(options.wafile)
    wa = wastore.cursor()

    # gets all the chat sessions
    try:
        wa.execute("SELECT * FROM wa_contacts WHERE is_whatsapp_user = 1 ")
        for chats in wa:
            curr_chat = Chatsession(chats[0],chats[7],chats[1],"",chats[4])
            chat_session_list.append(curr_chat)
    except sqlite3.Error, msg:
        print 'Error: {}'.format(msg)
        sys.exit(1)

    # for each chat session, gets all messages
    for chats in chat_session_list:
        try:
            c.execute("SELECT * FROM messages WHERE key_remote_jid=? ORDER BY _id ASC;", [chats.contact_id])
            for msgs in c:
                curr_message = Message(msgs[0],msgs[7],msgs[6],msgs[16],chats.contact_id,"",msgs[2],msgs[4],msgs[8],msgs[10],msgs[11],msgs[13],msgs[14])
                chats.msg_list.append(curr_message)

        except sqlite3.Error, msg:
            print 'Error: {}'.format(msg)
            sys.exit(1)

    # gets the db owner id
    # doesn't work anymore since new database structure on Android
    '''
    try:
        c.execute("SELECT key_remote_jid FROM messages WHERE key_from_me='1'")
        try:
            owner = (c.fetchone()[0]).split('/')[0]
        except:
            owner = "NotAvailable"
    except sqlite3.Error, msg:
        print 'Error: {}'.format(msg)
        sys.exit(1)
    '''

    owner = options.infile

    # OUTPUT
    wfile = file('%s.html' % owner,'wb')
    # writes page header
    wfile.write('<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"\n')
    wfile.write('"http://www.w3.org/TR/html4/loose.dtd">\n')
    wfile.write('<html><head><title>{}</title>\n'.format(owner))
    wfile.write('<meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>\n')
    wfile.write('<meta name="GENERATOR" content="WhatsApp Extractor v1.0">\n')
    # adds page style
    wfile.write(css_style)
    
    # adds javascript to make the tables sortable
    wfile.write('\n<script type="text/javascript">\n')
    wfile.write(popups)
    wfile.write(sortable)
    wfile.write('</script>\n\n')
    wfile.write('</head><body>\n')

    # writes 1st table header "CHAT SESSION"
    wfile.write('<a name="top"></a><h2>{} chat sessions</h2>\n'.format(owner))
    wfile.write('<table class="sortable" id="chatsession" border="1" cellpadding="2" cellspacing="0">\n')
    wfile.write('<thead>\n')
    wfile.write('<tr>\n')
    wfile.write('<th>PK</th>\n')
    wfile.write('<th>Contact Name</th>\n')
    wfile.write('<th>Contact ID</th>\n')
    wfile.write('<th>Status</th>\n')
    #wfile.write('<th># Messages</th>\n')
    wfile.write('</tr>\n')
    wfile.write('</thead>\n')
    
    # writes 1st table content
    wfile.write('<tbody>\n')
    for i in chat_session_list:
        contactname = convertsmileys ( i.contact_name )
        wfile.write('<tr>\n')
        wfile.write('<td>{}</td>\n'.format(i.pk_cs))
        wfile.write('<td class="contact"><a href="#{}">{}</a></td>\n'.format(i.contact_name,contactname))
        wfile.write('<td class="contact">{}</td>\n'.format(i.contact_id))
        wfile.write('<td>{}</td>\n'.format(i.contact_status))
        # wfile.write('<td>{}</td>\n'.format(i.contact_msg_count))
        wfile.write('</tr>\n')
    wfile.write('</tbody>\n')
    # writes 1st table footer
    wfile.write('</table>\n')

    # writes a table for each chat session
    for i in chat_session_list:
        contactname = convertsmileys ( i.contact_name )
        wfile.write('<h3>Chat session <a href="#top">#</a> {}: <a name="{}">{}</a></h3>\n'.format(i.pk_cs, i.contact_name, contactname))
        wfile.write('<table class="sortable" id="msg_{}" border="1" cellpadding="2" cellspacing="0">\n'.format(i.contact_id.split('@')[0]))
        wfile.write('<thead>\n')
        wfile.write('<tr>\n')
        wfile.write('<th>PK</th>\n')
        wfile.write('<th>Chat</th>\n')
        wfile.write('<th>Msg date</th>\n')
        wfile.write('<th>From</th>\n')
        wfile.write('<th>Msg content</th>\n')
        wfile.write('<th>Status</th>\n')
        wfile.write('<th>Type</th>\n')
        wfile.write('<th>Size</th>\n')
        wfile.write('</tr>\n')
        wfile.write('</thead>\n')

        # writes table content
        wfile.write('<tbody>\n')
        for y in i.msg_list:
            if y.from_me == 1:
                if y.status == 6:
                    wfile.write('<tr class="newgroupname">\n')
                    for n in chat_session_list:
                        if n.contact_id == y.contact_from:
                            y.contact_from = n.contact_name
                else:
                    wfile.write('<tr class="me">\n')
                    y.contact_from = "me"
            else:
                for n in chat_session_list:
                    if n.contact_id == y.contact_from:
                        y.contact_from = n.contact_name
                wfile.write('<tr class="other">\n')
            wfile.write('<td>{}</td>\n'.format(y.pk_msg))
            if y.contact_to == "":
                y.contact_to = contactname
            try:
                #wfile.write('<td class="contact">{}</td>\n'.format(y.contact_to.split('/')[0]))
                wfile.write('<td class="contact">{}</td>\n'.format(y.contact_to))
            except:
                wfile.write('<td class="contact">N/A</td>\n')
            wfile.write('<td>{}</td>\n'.format(y.msg_date))
            try:
                wfile.write('<td class="contact">{}</td>\n'.format(y.contact_from.split('/')[0]))
            except:
                try:
                    wfile.write('<td class="contact">{}</td>\n'.format(y.contact_from))
                except:
                    wfile.write('<td class="contact">N/A</td>\n')
            date = str(y.msg_date)[:10]
            date = int(date.replace("-",""))
            if str(y.msg_text)[:3] == "/9j":
                if y.media_wa_type == "5":
                    try:
                        wfile.write('<td class="text"><a onclick="image(this.href);return(false);" target="media" href="https://maps.google.com/?q={},{}"><img src="data:image/jpg;base64,\n{}" alt="Bild"/></a></td>\n'.format(y.latitude, y.longitude, y.msg_text))
                    except:
                        wfile.write('<td class="text">GPS N/A</td>\n')
                else:
                    if y.media_wa_type == "3":
                        #Search for offline file with current date (+2 days) and known media size
                        linkvideo = findfile ("VID", y.media_size, date, 2)
                        try:
                            # wfile.write('<td class="text"><a href="{}"><img src="data:image/jpg;base64,\n{}" alt="Video"/></a> (Video)</td>\n'.format(y.media_url, y.msg_text))
                            wfile.write('<td class="text"><a onclick="media(this.href);return(false);" target="media" href="{}"><img src="data:image/jpg;base64,\n{}" alt="Video"/></a>&nbsp;|&nbsp;<a onclick="media(this.href);return(false);" target="media" href="{}">Video</a></td>\n'.format(y.media_url, y.msg_text, linkvideo))
                        except:
                            wfile.write('<td class="text">Video N/A</td>\n')
                    else:
                        #Search for offline file with current date (+2 days) and known media size                   
                        linkimage = findfile ("IMG", y.media_size, date, 2)
                        try:
                            wfile.write('<td class="text"><a onclick="image(this.href);return(false);" target="image" href="{}"><img src="data:image/jpg;base64,\n{}" alt="Bild"/></a>&nbsp;|&nbsp;<a onclick="image(this.href);return(false);" target="image" href="{}">Image</a></td>\n'.format(y.media_url, y.msg_text, linkimage))
                        except:
                            wfile.write('<td class="text">Bild N/A</td>\n')
            else:
                if y.media_wa_type == "2":
                    #Search for offline file with current date (+2 days) and known media size                    
                    linkaudio = findfile ("AUD", y.media_size, date, 2)
                    try:
                        # wfile.write('<td class="text"><a href="{}">Audio File</a></td>\n'.format(y.media_url))
                        wfile.write('<td class="text"><a onclick="media(this.href);return(false);" target="media" href="{}">Audio (online)</a>&nbsp;|&nbsp;<a onclick="media(this.href);return(false);" target="media" href="{}">Audio (offline)</a></td>\n'.format(y.media_url, linkaudio))
                    except:
                        wfile.write('<td class="text">Audio N/A</td>\n')
                else:
                    msgtext = convertsmileys ( y.msg_text )
                    msgtext = re.sub(r'(http[^\s\n\r]+)', r'<a onclick="image(this.href);return(false);" target="image" href="\1">\1</a>', msgtext)
                    msgtext = re.sub(r'((?<!\S)www\.[^\s\n\r]+)', r'<a onclick="image(this.href);return(false);" target="image" href="http://\1">\1</a>', msgtext)
                    msgtext = msgtext.replace ("\n", "<br>\n")
                    try:
                        #wfile.write('<td class="text">{}</td>\n'.format(y.msg_text.encode('latin-1')))
                        wfile.write('<td class="text">{}</td>\n'.format(msgtext))
                    except:
                        wfile.write('<td class="text">N/A</td>\n')                    
            wfile.write('<td class="text">{}</td>\n'.format(y.status))
            wfile.write('<td class="text">{}</td>\n'.format(y.media_wa_type))
            wfile.write('<td class="text">{}</td>\n'.format(y.media_size))
            wfile.write('</tr>\n')
        wfile.write('</tbody>\n')
        # writes 1st table footer
        wfile.write('</table>\n')

    # writes page footer        
    wfile.write('</body></html>\n')
    wfile.close()

    #END
    webbrowser.open('%s.html' % owner)            
 

##### GLOBAL variables #####
flistvid = {}
flistaud = {}
flistimg = {}
css_style = """
<style type="text/css">
body {
    font-family: calibri;
}
h1 {
    font-style:italic;
}
h2 {
    font-style:italic;
}
h3 {
    font-style:italic;
}
table {
    text-align: center;
}
th {
    font-style:italic;
}
td.text {
    width: 600px;
    text-align: left;
}
td.contact {
    width: 250px;
}
tr.even {
    background-color: #ddd;
}
tr.me {
    background-color: #8f8;
}
tr.other {
    background-color: #fff;
}
tr.newgroupname {
    background-color: #FFCC33;
}
</style>
"""

popups = """
function media( url )
{
  if (typeof(newwindow) !== "undefined" && !newwindow.closed)
  {
    newwindow.close();
  } else
  {
  }
  newwindow=window.open("about:blank", "media", "menubar=0,location=1");
  newwindow.document.write('<html><body><embed src="' + url + '" width="800" height="600" /></body></html>');
  //newwindow.document.write('<video src="' + url + '" controls="controls">video doesn't work</video>');
}

function image( url )
{
  if (typeof(imagewindow) !== "undefined" && !imagewindow.closed)
  {
    imagewindow=window.open(url,'image','menubar=0,location=1');
    imagewindow.focus();
  } else
  {
    imagewindow=window.open(url,'image','menubar=0,location=1');
  }
}
"""

sortable = """
/*
Table sorting script  by Joost de Valk, check it out at http://www.joostdevalk.nl/code/sortable-table/.
Based on a script from http://www.kryogenix.org/code/browser/sorttable/.
Distributed under the MIT license: http://www.kryogenix.org/code/browser/licence.html .

Copyright (c) 1997-2007 Stuart Langridge, Joost de Valk.

Version 1.5.7
*/

/* You can change these values */
var image_path = "sort-table/";
var image_up = "arrow-up.gif";
var image_down = "arrow-down.gif";
var image_none = "arrow-none.gif";
var europeandate = true;
var alternate_row_colors = true;

/* Don't change anything below this unless you know what you're doing */
addEvent(window, "load", sortables_init);

var SORT_COLUMN_INDEX;
var thead = false;

function sortables_init() {
	// Find all tables with class sortable and make them sortable
	if (!document.getElementsByTagName) return;
	tbls = document.getElementsByTagName("table");
	for (ti=0;ti<tbls.length;ti++) {
		thisTbl = tbls[ti];
		if (((' '+thisTbl.className+' ').indexOf("sortable") != -1) && (thisTbl.id)) {
			ts_makeSortable(thisTbl);
		}
	}
}

function ts_makeSortable(t) {
	if (t.rows && t.rows.length > 0) {
		if (t.tHead && t.tHead.rows.length > 0) {
			var firstRow = t.tHead.rows[t.tHead.rows.length-1];
			thead = true;
		} else {
			var firstRow = t.rows[0];
		}
	}
	if (!firstRow) return;
	
	// We have a first row: assume it's the header, and make its contents clickable links
	for (var i=0;i<firstRow.cells.length;i++) {
		var cell = firstRow.cells[i];
		var txt = ts_getInnerText(cell);
		if (cell.className != "unsortable" && cell.className.indexOf("unsortable") == -1) {
			cell.innerHTML = '<a href="#" class="sortheader" onclick="ts_resortTable(this, '+i+');return false;">'+txt+'<span class="sortarrow">&nbsp;&nbsp;<img src="'+ image_path + image_none + '" alt="&darr;"/></span></a>';
		}
	}
	if (alternate_row_colors) {
		alternate(t);
	}
}

function ts_getInnerText(el) {
	if (typeof el == "string") return el;
	if (typeof el == "undefined") { return el };
	if (el.innerText) return el.innerText;	//Not needed but it is faster
	var str = "";
	
	var cs = el.childNodes;
	var l = cs.length;
	for (var i = 0; i < l; i++) {
		switch (cs[i].nodeType) {
			case 1: //ELEMENT_NODE
				str += ts_getInnerText(cs[i]);
				break;
			case 3:	//TEXT_NODE
				str += cs[i].nodeValue;
				break;
		}
	}
	return str;
}

function ts_resortTable(lnk, clid) {
	var span;
	for (var ci=0;ci<lnk.childNodes.length;ci++) {
		if (lnk.childNodes[ci].tagName && lnk.childNodes[ci].tagName.toLowerCase() == 'span') span = lnk.childNodes[ci];
	}
	var spantext = ts_getInnerText(span);
	var td = lnk.parentNode;
	var column = clid || td.cellIndex;
	var t = getParent(td,'TABLE');
	// Work out a type for the column
	if (t.rows.length <= 1) return;
	var itm = "";
	var i = 0;
	while (itm == "" && i < t.tBodies[0].rows.length) {
		var itm = ts_getInnerText(t.tBodies[0].rows[i].cells[column]);
		itm = trim(itm);
		if (itm.substr(0,4) == "<!--" || itm.length == 0) {
			itm = "";
		}
		i++;
	}
	if (itm == "") return; 
	sortfn = ts_sort_caseinsensitive;
	if (itm.match(/^\d\d[\/\.-][a-zA-z][a-zA-Z][a-zA-Z][\/\.-]\d\d\d\d$/)) sortfn = ts_sort_date;
	if (itm.match(/^\d\d[\/\.-]\d\d[\/\.-]\d\d\d{2}?$/)) sortfn = ts_sort_date;
	if (itm.match(/^-?[$ۢ]\d/)) sortfn = ts_sort_numeric;
	if (itm.match(/^-?(\d+[,\.]?)+(E[-+][\d]+)?%?$/)) sortfn = ts_sort_numeric;
	SORT_COLUMN_INDEX = column;
	var firstRow = new Array();
	var newRows = new Array();
	for (k=0;k<t.tBodies.length;k++) {
		for (i=0;i<t.tBodies[k].rows[0].length;i++) { 
			firstRow[i] = t.tBodies[k].rows[0][i]; 
		}
	}
	for (k=0;k<t.tBodies.length;k++) {
		if (!thead) {
			// Skip the first row
			for (j=1;j<t.tBodies[k].rows.length;j++) { 
				newRows[j-1] = t.tBodies[k].rows[j];
			}
		} else {
			// Do NOT skip the first row
			for (j=0;j<t.tBodies[k].rows.length;j++) { 
				newRows[j] = t.tBodies[k].rows[j];
			}
		}
	}
	newRows.sort(sortfn);
	if (span.getAttribute("sortdir") == 'down') {
			ARROW = '&nbsp;&nbsp;<img src="'+ image_path + image_down + '" alt="&darr;"/>';
			newRows.reverse();
			span.setAttribute('sortdir','up');
	} else {
			ARROW = '&nbsp;&nbsp;<img src="'+ image_path + image_up + '" alt="&uarr;"/>';
			span.setAttribute('sortdir','down');
	} 
    // We appendChild rows that already exist to the tbody, so it moves them rather than creating new ones
    // don't do sortbottom rows
    for (i=0; i<newRows.length; i++) { 
		if (!newRows[i].className || (newRows[i].className && (newRows[i].className.indexOf('sortbottom') == -1))) {
			t.tBodies[0].appendChild(newRows[i]);
		}
	}
    // do sortbottom rows only
    for (i=0; i<newRows.length; i++) {
		if (newRows[i].className && (newRows[i].className.indexOf('sortbottom') != -1)) 
			t.tBodies[0].appendChild(newRows[i]);
	}
	// Delete any other arrows there may be showing
	var allspans = document.getElementsByTagName("span");
	for (var ci=0;ci<allspans.length;ci++) {
		if (allspans[ci].className == 'sortarrow') {
			if (getParent(allspans[ci],"table") == getParent(lnk,"table")) { // in the same table as us?
				allspans[ci].innerHTML = '&nbsp;&nbsp;<img src="'+ image_path + image_none + '" alt="&darr;"/>';
			}
		}
	}		
	span.innerHTML = ARROW;
	alternate(t);
}

function getParent(el, pTagName) {
	if (el == null) {
		return null;
	} else if (el.nodeType == 1 && el.tagName.toLowerCase() == pTagName.toLowerCase()) {
		return el;
	} else {
		return getParent(el.parentNode, pTagName);
	}
}

function sort_date(date) {	
	// y2k notes: two digit years less than 50 are treated as 20XX, greater than 50 are treated as 19XX
	dt = "00000000";
	if (date.length == 11) {
		mtstr = date.substr(3,3);
		mtstr = mtstr.toLowerCase();
		switch(mtstr) {
			case "jan": var mt = "01"; break;
			case "feb": var mt = "02"; break;
			case "mar": var mt = "03"; break;
			case "apr": var mt = "04"; break;
			case "may": var mt = "05"; break;
			case "jun": var mt = "06"; break;
			case "jul": var mt = "07"; break;
			case "aug": var mt = "08"; break;
			case "sep": var mt = "09"; break;
			case "oct": var mt = "10"; break;
			case "nov": var mt = "11"; break;
			case "dec": var mt = "12"; break;
			// default: var mt = "00";
		}
		dt = date.substr(7,4)+mt+date.substr(0,2);
		return dt;
	} else if (date.length == 10) {
		if (europeandate == false) {
			dt = date.substr(6,4)+date.substr(0,2)+date.substr(3,2);
			return dt;
		} else {
			dt = date.substr(6,4)+date.substr(3,2)+date.substr(0,2);
			return dt;
		}
	} else if (date.length == 8) {
		yr = date.substr(6,2);
		if (parseInt(yr) < 50) { 
			yr = '20'+yr; 
		} else { 
			yr = '19'+yr; 
		}
		if (europeandate == true) {
			dt = yr+date.substr(3,2)+date.substr(0,2);
			return dt;
		} else {
			dt = yr+date.substr(0,2)+date.substr(3,2);
			return dt;
		}
	}
	return dt;
}

function ts_sort_date(a,b) {
	dt1 = sort_date(ts_getInnerText(a.cells[SORT_COLUMN_INDEX]));
	dt2 = sort_date(ts_getInnerText(b.cells[SORT_COLUMN_INDEX]));
	
	if (dt1==dt2) {
		return 0;
	}
	if (dt1<dt2) { 
		return -1;
	}
	return 1;
}
function ts_sort_numeric(a,b) {
	var aa = ts_getInnerText(a.cells[SORT_COLUMN_INDEX]);
	aa = clean_num(aa);
	var bb = ts_getInnerText(b.cells[SORT_COLUMN_INDEX]);
	bb = clean_num(bb);
	return compare_numeric(aa,bb);
}
function compare_numeric(a,b) {
	var a = parseFloat(a);
	a = (isNaN(a) ? 0 : a);
	var b = parseFloat(b);
	b = (isNaN(b) ? 0 : b);
	return a - b;
}
function ts_sort_caseinsensitive(a,b) {
	aa = ts_getInnerText(a.cells[SORT_COLUMN_INDEX]).toLowerCase();
	bb = ts_getInnerText(b.cells[SORT_COLUMN_INDEX]).toLowerCase();
	if (aa==bb) {
		return 0;
	}
	if (aa<bb) {
		return -1;
	}
	return 1;
}
function ts_sort_default(a,b) {
	aa = ts_getInnerText(a.cells[SORT_COLUMN_INDEX]);
	bb = ts_getInnerText(b.cells[SORT_COLUMN_INDEX]);
	if (aa==bb) {
		return 0;
	}
	if (aa<bb) {
		return -1;
	}
	return 1;
}
function addEvent(elm, evType, fn, useCapture)
// addEvent and removeEvent
// cross-browser event handling for IE5+,	NS6 and Mozilla
// By Scott Andrew
{
	if (elm.addEventListener){
		elm.addEventListener(evType, fn, useCapture);
		return true;
	} else if (elm.attachEvent){
		var r = elm.attachEvent("on"+evType, fn);
		return r;
	} else {
		alert("Handler could not be removed");
	}
}
function clean_num(str) {
	str = str.replace(new RegExp(/[^-?0-9.]/g),"");
	return str;
}
function trim(s) {
	return s.replace(/^\s+|\s+$/g, "");
}
function alternate(table) {
	// Take object table and get all it's tbodies.
	var tableBodies = table.getElementsByTagName("tbody");
	// Loop through these tbodies
	for (var i = 0; i < tableBodies.length; i++) {
		// Take the tbody, and get all it's rows
		var tableRows = tableBodies[i].getElementsByTagName("tr");
		// Loop through these rows
		// Start at 1 because we want to leave the heading row untouched
		for (var j = 0; j < tableRows.length; j++) {
			// Check if j is even, and apply classes for both possible results
			if ( (j % 2) == 0  ) {
				if ( !(tableRows[j].className.indexOf('odd') == -1) ) {
					tableRows[j].className = tableRows[j].className.replace('odd', 'even');
				} else {
					if ( tableRows[j].className.indexOf('even') == -1 ) {
						tableRows[j].className += " even";
					}
				}
			} else {
				if ( !(tableRows[j].className.indexOf('even') == -1) ) {
					tableRows[j].className = tableRows[j].className.replace('even', 'odd');
				} else {
					if ( tableRows[j].className.indexOf('odd') == -1 ) {
						tableRows[j].className += " odd";
					}
				}
			} 
		}
	}
}
"""

if __name__ == '__main__':
    main(sys.argv[1:])

